﻿using Steamworks;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

public class PlayerCompass : Mod
{
    RawImage CompassImage;
    Text CompassDirectionText;
    float compassUnit = 1.06f;
    GameObject PlayerEntry;
    GameObject canvas;
    GameObject playerparent;
    AssetBundle asset;

    public void Start()
    {
        asset = AssetBundle.LoadFromMemory(GetEmbeddedFileBytes("playercompass.assets"));
        canvas = Instantiate(asset.LoadAsset<GameObject>("CompassCanvas"), transform);
        playerparent = canvas.transform.Find("PlayerParent").gameObject;
        CompassImage = canvas.transform.Find("Background").Find("CompassTexture").GetComponent<RawImage>();
        CompassDirectionText = canvas.transform.Find("MiddleText").GetComponent<Text>();
        PlayerEntry = asset.LoadAsset<GameObject>("PlayerEntry");

        Debug.Log("Mod PlayerCompass has been loaded!");
    }

    public void Update()
    {
        if (RAPI.IsCurrentSceneMainMenu() || RAPI.GetLocalPlayer() == null) { canvas.SetActive(false); return; }
        canvas.SetActive(true);
        List<Network_Player> connectedPlayers = ComponentManager<Raft_Network>.Value.remoteUsers.Values.ToList();

        connectedPlayers.ForEach(player =>
        {
            if (player.IsLocalPlayer) { return; }
            Transform text = playerparent.transform.Find("Player_" + player.steamID.m_SteamID.ToString());
            if (text == null)
            {
                text = Instantiate(PlayerEntry, playerparent.transform).transform;
                text.name = "Player_" + player.steamID.m_SteamID.ToString();
            }

            text.GetComponent<RectTransform>().anchoredPosition = GetPosOnCompass(player.transform.position);
            text.GetComponent<Text>().text = player.characterSettings.Name + "\n<b>" + Mathf.RoundToInt(Vector3.Distance(RAPI.GetLocalPlayer().transform.position, player.transform.position)) + "m</b>";
        });

        foreach (Transform child in playerparent.transform)
        {
            if (!connectedPlayers.Where(p => "Player_" + p.steamID.m_SteamID.ToString() == child.name).Any())
            {
                Destroy(child.gameObject);
            }
        }

        Vector3 forward = RAPI.GetLocalPlayer().transform.forward;
        forward.y = 0;
        float headingAngle = Quaternion.LookRotation(forward).eulerAngles.y;
        CompassImage.uvRect = new Rect(headingAngle / 360, 0, 1, 1); //absolute heading (added)
        headingAngle = 5 * (Mathf.RoundToInt(headingAngle / 5.0f));
        int displayangle;
        displayangle = Mathf.RoundToInt(headingAngle);
        switch (displayangle)
        {
            case 0:
                CompassDirectionText.text = "N";
                break;
            case 360:
                CompassDirectionText.text = "N";
                break;
            case 45:
                //Do this
                CompassDirectionText.text = "NE";
                break;
            case 90:
                CompassDirectionText.text = "E";
                break;
            case 130:
                CompassDirectionText.text = "SE";
                break;
            case 180:
                CompassDirectionText.text = "S";
                break;
            case 225:
                CompassDirectionText.text = "SW";
                break;
            case 270:
                CompassDirectionText.text = "W";
                break;
            default:
                CompassDirectionText.text = headingAngle.ToString();
                break;
        }
    }

    Vector2 GetPosOnCompass(Vector3 pos)
    {
        Vector2 playerPos = new Vector2(RAPI.GetLocalPlayer().transform.position.x, RAPI.GetLocalPlayer().transform.position.z);
        Vector2 playerFwd = new Vector2(RAPI.GetLocalPlayer().transform.forward.x, RAPI.GetLocalPlayer().transform.forward.z);
        float angle = Vector2.SignedAngle(new Vector2(pos.x, pos.z) - playerPos, playerFwd);
        return new Vector2(compassUnit * angle, -105f);
    }

    public void OnModUnload()
    {
        asset.Unload(true);
        Debug.Log("PlayerCompass has been unloaded!");
    }
}